'use strict';

/**
 * services.cordova Module
 *
 * General Cordova services wrapper module
 */
angular.module('starter.network', [])

/**
 * CordovaNetwork AngularJS service
 *
 * com.apache.cordova.network-information wrapper for AngularJS with fallback for testing without Cordova
 */
.service('CordovaNetwork', ['$ionicPlatform', '$q', '$rootScope', function($ionicPlatform, $q, $rootScope) {
  // Get Cordova's global Connection object or emulate a smilar one
  var Connection = window.Connection || {
    "CELL"     : "cellular",
    "CELL_2G"  : "2g",
    "CELL_3G"  : "3g",
    "CELL_4G"  : "4g",
    "ETHERNET" : "ethernet",
    "NONE"     : "none",
    "UNKNOWN"  : "unknown",
    "WIFI"     : "wifi"
  };

  var asyncGetConnection = function () {
    var q = $q.defer();
    $ionicPlatform.ready(function () {
      if(navigator.connection) {
        q.resolve(navigator.connection);
      } else {
        q.reject('navigator.connection is not defined');
      }
    });
    return q.promise;
  };

  // These services are written with full Ionic Framework integration.
  $ionicPlatform.ready(function () {
    document.addEventListener('offline', function () {
      $rootScope.$broadcast('Cordova.NetworkStatus.Offline');
    }, false);
    document.addEventListener('online', function () {
      $rootScope.$broadcast('Cordova.NetworkStatus.Online');
    }, false);
  });

  return {
    isOnline: function () {
      return asyncGetConnection().then(function(networkConnection) {
        switch (networkConnection.type) {
          case Connection.ETHERNET:
          case Connection.WIFI:
          case Connection.CELL_2G:
          case Connection.CELL_3G:
          case Connection.CELL_4G:
          case Connection.CELL:
            return true;
        }
//        TODO uncomment if issues occur with galaxy s3
//        switch (networkConnection.type.toLowerCase()) {
//          case 'ethernet':
//          case 'wifi':
//          case 'cell_2g':
//          case 'cell_3g':
//          case 'cell_4g':
//          case '2g':
//          case '3g':
//          case '4g':
//          case 'cell':
//          case 'cellular':
//            return true;
//        }
        return false;
      }, function(){return false;});
    }
  };
}]);
