angular.module('starter.services', [])

.factory('Lines', function($http, $q) {
    var VERSION_DATE_KEY = "version";
    var FAVORITES_KEY = "favorites";
    var LINES_KEY = "lines";
    var VERSION_URL = "http://www.gspns.rs/feeds/red-voznje";
    var SCHEDULE_URL = "http://www.gspns.rs/feeds/polasci";
    Array.prototype.inArray = function(comparer) {
      for(var i=0; i < this.length; i++) {
        if(comparer(this[i])) {
          return true;
        }
      }
      return false;
    };

    Array.prototype.getBy = function(comparer) {
      for(var i=0; i < this.length; i++) {
        if(comparer(this[i])) {
          return this[i];
        }
      }
      return undefined;
    };

    Array.prototype.pushIfNotExist = function(element, comparer) {
      if (!this.inArray(comparer)) {
        this.push(element);
      }
    };

    Array.prototype.pushIfNotExist = function(element) {
      if (this.indexOf(element) < 0) {
        this.push(element);
      }
    };

  function convert(input, favs){
    var converted = [];
    var compareId = function(e){
      return e.id == entryId;
        };
    var compareHour = function(e){
          return e.h == hour;
        };
    var length = input.length;
    for(var i = 0; i < length; i++ ) {
      var entry = input[i];
      var entryId;
      // hack to use 3A line as a separate one
      if(entry.OZ === '3A' || entry.OZ === '6A') {
         entryId = entry.OZ.toLowerCase();
      } else {
         entryId = entry.OZ.replace(/\D+/g, '');
      }
      var object = converted.getBy(compareId);
      if(object === undefined){
        object = {id:entryId,type:entry.PR, r:{a:[],b:[]},s:{a:[],b:[]},n:{a:[],b:[]}};
        converted.push(object);
      }
      object.fav = favs.indexOf(object.id) >= 0;
      var day;
      switch(entry.KA) {
        case "R":
          day = object.r;
          break;
        case "S":
          day = object.s;
          break;
        case "N":
          day = object.n;
          break;
      }
      var direction;
      if(entry.SM == 'A') {
        direction = day.a;
      } else if(entry.SM == 'B') {
        direction = day.b;
      }
      var split = entry.PO.split(':');
      var hour = split[0];
      var min = split[1];

      var dirHour = direction.getBy(compareHour);
      if(dirHour === undefined) {
        dirHour = {h:hour,data:[]};
        direction.push(dirHour);
      }
      dirHour.data.push({m:min,o:entry.KO,t:entry.TI});
      if(entry.TI === 'Z') {
        day.z = true;
      }
      if(entry.TI === 'C'){
        day.c = true;
      }
      if(entry.KO !== undefined && entry.KO.length > 0) {
        if(day.l === undefined) {
          day.l = [];
        }
        day.l.pushIfNotExist(entry.KO);
      }
    }
    converted.sort(function(a,b){
      var as = a.id.replace( /\D+/g, '');
      var bs = b.id.replace( /\D+/g, '');
      if(as == bs) {
        return a.id.length - b.id.length;
      } else {
        return as - bs;
      }
    });
    return converted;
  }

  var lines;
  if(window.localStorage[LINES_KEY] === undefined) {
      lines = [];
  } else {
      var lines = JSON.parse(window.localStorage[LINES_KEY]);
  }
  var favorites;
  if(window.localStorage[FAVORITES_KEY] === undefined) {
    favorites = [];
  } else {
    favorites = JSON.parse(window.localStorage[FAVORITES_KEY]);
  }

  return {
    lines: function(){
      return lines;
    },
    favorites: function(){
      return favorites;
    },
    toggleFavourite: function(line){
      var id = line.id;
      var index = favorites.indexOf(id);
      if (index > -1) {
        favorites.splice(index, 1);
        line.fav = false;
      } else {
        favorites.pushIfNotExist(id, function(e) {
            return e === id;
        });
        line.fav = true;
      }
      window.localStorage[FAVORITES_KEY] = JSON.stringify(favorites);
      window.localStorage[LINES_KEY] = JSON.stringify(lines);
    },
    hasSchedule: function() {
        return window.localStorage[VERSION_DATE_KEY] !== undefined;
    },
    checkForUpdates: function() {
      var q = $q.defer();
      var oldDate = window.localStorage[VERSION_DATE_KEY];
      if (oldDate === undefined) {
          q.resolve(true);
      } else {
          $http.get(VERSION_URL).then(function(response){
              var date = response.data[0].datum;
//              q.resolve(true);
              q.resolve(date > oldDate);
          });
      }
      return q.promise;
    },
    updateSchedule: function() {
      var q = $q.defer();
      $http.get(SCHEDULE_URL).then(function(response){
        try{
          lines = convert(response.data, favorites);
          window.localStorage[LINES_KEY] = JSON.stringify(lines);
        } catch(err) {
          q.reject();
        }
        $http.get(VERSION_URL).then(function(responseDate){
          try{
            var latestDate = responseDate.data[0].datum;
            window.localStorage[VERSION_DATE_KEY] = latestDate;
            q.resolve(lines);
          } catch(err) {
            q.reject();
          }
        }, function(){q.reject();});
      }, function() {q.reject();});
      return q.promise;
    }
};
})

.factory('Time', function($interval) {
  var now  = new Date();
  var currentHour = now.getHours();
  $interval(function(){
    now = new Date();
    currentHour = now.getHours();
  },60000);

  return {
    currentDay: function() {
      switch(now.getDay()) {
        case 0:
          return 'sunday';
        case 6:
          return 'saturday';
        default:
          return 'work-day';
      }
    },
    currentHour: function() {
      return currentHour;
    }
  };
})

.factory('Updates', function(Lines, CordovaNetwork, $q, $rootScope) {
  var state = "init";
  var q = $q.defer();

  $rootScope.$on('Cordova.NetworkStatus.Online',
                 function(){
                  if(state == 'offline') {
                    state = 'init';
                    checkForUpdates();
                  }
                });

  var checkForUpdates = function() {
    if(state != 'init') {
      return;
    }
    Lines.checkForUpdates().then(function(hasUpdates) {
      if(hasUpdates){
        state = 'hasUpdates';
        q.resolve(true);
      } else {
        state = 'noUpdates';
      }
    });
  };

  CordovaNetwork.isOnline().then(function(isConnected) {
    if(isConnected) {
      checkForUpdates();
    } else {
      state = 'offline';
    }
  });

  return {
    check: function() {
      return q.promise;
    },
    cancel: function() {
        state = 'canceled';
    },
    update: function() {
      state = 'updating';
      var u = $q.defer();
      Lines.updateSchedule().then(function(lines){
        state = 'updated';
        u.resolve(lines);
      }, function() {
        state = 'error';
        u.reject();
      });
      return u.promise;
    },
    state: function() {
      return state;
    }
  };
})
;
